@@ -1,6 +1,6 @@
 <?php declare(strict_types = 0);
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -104,6 +104,11 @@ class CControllerMenuPopup extends CController {
 		$validator = new CNewValidator($this->input['data'], $rules);
 		array_map('error', $validator->getAllErrors());
 
+		if (array_key_exists('backurl', $this->input['data'])
+				&& !CHtmlUrlValidator::validateSameSite($this->input['data']['backurl'])) {
+			throw new CAccessDeniedException();
+		}
+
 		return !$validator->isError() && !$validator->isErrorFatal();
 	}
 
@@ -163,9 +168,10 @@ class CControllerMenuPopup extends CController {
 
 		$db_hosts = $has_goto
 			? API::Host()->get([
-				'output' => ['hostid', 'status'],
+				'output' => ['hostid', 'status', 'flags'],
 				'selectGraphs' => API_OUTPUT_COUNT,
 				'selectHttpTests' => API_OUTPUT_COUNT,
+				'selectDashboards' => API_OUTPUT_COUNT,
 				'hostids' => $data['hostid']
 			])
 			: API::Host()->get([
@@ -244,9 +250,10 @@ class CControllerMenuPopup extends CController {
 
 			if ($has_goto) {
 				$menu_data['showGraphs'] = (bool) $db_host['graphs'];
-				$menu_data['showDashboards'] = (bool) getHostDashboards($data['hostid']);
+				$menu_data['showDashboards'] = (bool) $db_host['dashboards'];
 				$menu_data['showWeb'] = (bool) $db_host['httpTests'];
 				$menu_data['isWriteable'] = $rw_hosts;
+				$menu_data['isDiscovered'] = $db_host['flags'] == ZBX_FLAG_DISCOVERY_CREATED;
 				$menu_data['showTriggers'] = ($db_host['status'] == HOST_STATUS_MONITORED);
 				if (array_key_exists('severity_min', $data)) {
 					$menu_data['severities'] = array_column(
@@ -353,8 +360,9 @@ class CControllerMenuPopup extends CController {
 	 */
 	private static function getMenuDataItemPrototype(array $data) {
 		$db_item_prototypes = API::ItemPrototype()->get([
-			'output' => ['name', 'key_'],
+			'output' => ['name', 'key_', 'value_type', 'flags'],
 			'selectDiscoveryRule' => ['itemid'],
+			'selectDiscoveryRulePrototype' => ['itemid'],
 			'selectHosts' => ['host'],
 			'selectTriggers' => ['triggerid', 'description'],
 			'itemids' => $data['itemid']
@@ -362,6 +370,7 @@ class CControllerMenuPopup extends CController {
 
 		if ($db_item_prototypes) {
 			$db_item_prototype = $db_item_prototypes[0];
+			$parent_lld = $db_item_prototype['discoveryRule'] ?: $db_item_prototype['discoveryRulePrototype'];
 
 			$menu_data = [
 				'type' => 'item_prototype',
@@ -369,9 +378,11 @@ class CControllerMenuPopup extends CController {
 				'itemid' => $data['itemid'],
 				'name' => $db_item_prototype['name'],
 				'key' => $db_item_prototype['key_'],
+				'is_binary_value_type' => $db_item_prototype['value_type'] == ITEM_VALUE_TYPE_BINARY,
+				'is_discovered_prototype' => $db_item_prototype['flags'] & ZBX_FLAG_DISCOVERY_CREATED,
 				'hostid' => $db_item_prototype['hosts'][0]['hostid'],
 				'host' => $db_item_prototype['hosts'][0]['host'],
-				'parent_discoveryid' => $db_item_prototype['discoveryRule']['itemid'],
+				'parent_discoveryid' => $parent_lld['itemid'],
 				'trigger_prototypes' => $db_item_prototype['triggers']
 			];
 
