@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -378,7 +378,9 @@ calendar.prototype = {
 			new_val = this.sdt.format(this.date_time_format);
 
 		if ($input.val() != new_val) {
-			$input.val(new_val).trigger('change');
+			$input.val(new_val);
+			$input.trigger('change');
+			$input[0].dispatchEvent(new CustomEvent('field.change', {detail: {source_fields: [$input.attr('name')]}}));
 		}
 	},
 
