@@ -1,6 +1,6 @@
 <?php declare(strict_types = 0);
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -21,7 +21,7 @@ use CButton,
 	CButtonLink,
 	CCheckBox,
 	CCol,
-	CColor,
+	CColorPicker,
 	CDiv,
 	CFormField,
 	CFormGrid,
@@ -219,8 +219,11 @@ class CWidgetFieldDataSetView extends CWidgetFieldView {
 				]))->addClass('js-items-multiselect');
 			}
 
-			$dataset_head[] = (new CColor($field_name.'['.$row_num.'][color]', $value['color']))
-				->appendColorPickerJs(false);
+			$dataset_head[] = (new CColorPicker($field_name.'['.$row_num.'][color]',
+				$field_name.'['.$row_num.'][color_palette]')
+			)
+				->setPalette($value['color_palette'] ?? null)
+				->setColor($value['color'] ?? null);
 
 			if ($host_pattern_field !== null) {
 				$dataset_head[] = $host_pattern_field;
@@ -303,7 +306,7 @@ class CWidgetFieldDataSetView extends CWidgetFieldView {
 			}
 
 			$override_host_html[] = implode('', $override_host_field_view->getTemplates());
-			$override_host_html[] = (new CScriptTag($override_host_field_view->getJavaScript()))->setOnDocumentReady();
+			$override_host_html[] = new CScriptTag($override_host_field_view->getJavaScript());
 		}
 
 		return (new CListItem([
@@ -516,9 +519,7 @@ class CWidgetFieldDataSetView extends CWidgetFieldView {
 				->addClass('table-col-handle')
 				->addClass(ZBX_STYLE_TD_DRAG_ICON),
 			(new CCol(
-				(new CColor($this->field->getName().'['.$ds_num.'][color][]', $color,
-					'items_'.$ds_num.'_'.$row_num.'_color'
-				))->appendColorPickerJs(false)
+				(new CColorPicker($this->field->getName().'['.$ds_num.'][color][]'))->setColor($color)
 			))->addClass('table-col-color'),
 			(new CCol(new CSpan($row_num.':')))->addClass('table-col-no'),
 			(new CCol([
